@@ -21,7 +21,7 @@
 #define G_LOG_DOMAIN  "BAZAAR::ENTRY-CACHE"
 #define BAZAAR_MODULE "entry-cache"
 
-#define MAX_CONCURRENT_WRITES       4
+#define MAX_CONCURRENT_WRITES       16
 #define WATCH_CLEANUP_INTERVAL_MSEC 5000
 
 #include <malloc.h>
@@ -41,6 +41,8 @@ BZ_DEFINE_DATA (
     ongoing_task,
     OngoingTask,
     {
+      GWeakRef *self;
+
       DexScheduler *scheduler;
       DexPromise   *init;
 
@@ -60,6 +62,7 @@ BZ_DEFINE_DATA (
       BzGuard *writing_gate;
       GMutex   writing_mutex;
     },
+    BZ_RELEASE_DATA (self, bz_weak_release);
     BZ_RELEASE_DATA (scheduler, dex_unref);
     BZ_RELEASE_DATA (init, dex_unref);
     BZ_RELEASE_DATA (alive_hash, g_hash_table_unref);
@@ -81,7 +84,8 @@ struct _BzEntryCacheManager
 {
   GObject parent_instance;
 
-  guint64 max_memory_usage;
+  GMutex mutex;
+  guint  living_entries;
 
   DexScheduler *scheduler;
   guint64       memory_usage;
@@ -96,7 +100,7 @@ enum
 {
   PROP_0,
 
-  PROP_MAX_MEMORY_USAGE,
+  PROP_LIVING_ENTRIES,
 
   LAST_PROP
 };
@@ -112,6 +116,9 @@ watch_cb (DexFuture       *future,
 static DexFuture *
 watch_work_fiber (OngoingTaskData *task_data);
 
+static DexFuture *
+notify_props_fiber (GWeakRef *wr);
+
 BZ_DEFINE_DATA (
     living_entry,
     LivingEntry,
@@ -160,6 +167,8 @@ bz_entry_cache_manager_dispose (GObject *object)
 {
   BzEntryCacheManager *self = BZ_ENTRY_CACHE_MANAGER (object);
 
+  g_mutex_clear (&self->mutex);
+
   dex_clear (&self->scheduler);
   dex_clear (&self->watch_task);
   g_clear_pointer (&self->task_data, ongoing_task_data_unref);
@@ -177,8 +186,8 @@ bz_entry_cache_manager_get_property (GObject    *object,
 
   switch (prop_id)
     {
-    case PROP_MAX_MEMORY_USAGE:
-      g_value_set_uint64 (value, bz_entry_cache_manager_get_max_memory_usage (self));
+    case PROP_LIVING_ENTRIES:
+      g_value_set_uint (value, bz_entry_cache_manager_get_living_entries (self));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -193,11 +202,11 @@ bz_entry_cache_manager_set_property (GObject      *object,
 {
   BzEntryCacheManager *self = BZ_ENTRY_CACHE_MANAGER (object);
 
+  (void) self;
+
   switch (prop_id)
     {
-    case PROP_MAX_MEMORY_USAGE:
-      bz_entry_cache_manager_set_max_memory_usage (self, g_value_get_uint64 (value));
-      break;
+    case PROP_LIVING_ENTRIES:
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -212,12 +221,12 @@ bz_entry_cache_manager_class_init (BzEntryCacheManagerClass *klass)
   object_class->get_property = bz_entry_cache_manager_get_property;
   object_class->dispose      = bz_entry_cache_manager_dispose;
 
-  props[PROP_MAX_MEMORY_USAGE] =
-      g_param_spec_uint64 (
-          "max-memory-usage",
+  props[PROP_LIVING_ENTRIES] =
+      g_param_spec_uint (
+          "living-entries",
           NULL, NULL,
-          0, G_MAXUINT64, 0xccccccc,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+          0, G_MAXUINT, 0,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 }
@@ -231,10 +240,13 @@ bz_entry_cache_manager_init (BzEntryCacheManager *self)
   if (g_once_init_enter_pointer (&global_scheduler))
     g_once_init_leave_pointer (&global_scheduler, dex_thread_pool_scheduler_new ());
 
+  g_mutex_init (&self->mutex);
+
   self->scheduler    = dex_ref (global_scheduler);
   self->memory_usage = 0;
 
   task_data             = ongoing_task_data_new ();
+  task_data->self       = bz_track_weak (self);
   task_data->scheduler  = dex_ref (self->scheduler);
   task_data->init       = dex_promise_new ();
   task_data->alive_hash = g_hash_table_new_full (
@@ -265,22 +277,15 @@ bz_entry_cache_manager_new (void)
   return g_object_new (BZ_TYPE_ENTRY_CACHE_MANAGER, NULL);
 }
 
-guint64
-bz_entry_cache_manager_get_max_memory_usage (BzEntryCacheManager *self)
+guint
+bz_entry_cache_manager_get_living_entries (BzEntryCacheManager *self)
 {
+  g_autoptr (GMutexLocker) locker = NULL;
+
   g_return_val_if_fail (BZ_IS_ENTRY_CACHE_MANAGER (self), 0);
-  return self->max_memory_usage;
-}
 
-void
-bz_entry_cache_manager_set_max_memory_usage (BzEntryCacheManager *self,
-                                             guint64              max_memory_usage)
-{
-  g_return_if_fail (BZ_IS_ENTRY_CACHE_MANAGER (self));
-
-  self->max_memory_usage = max_memory_usage;
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_MAX_MEMORY_USAGE]);
+  locker = g_mutex_locker_new (&self->mutex);
+  return self->living_entries;
 }
 
 DexFuture *
@@ -373,28 +378,33 @@ bz_entry_cache_manager_enumerate_disk (BzEntryCacheManager *self)
 static DexFuture *
 write_task_fiber (WriteTaskData *data)
 {
-  OngoingTaskData *task_data           = data->task_data;
-  char            *unique_id_checksum  = data->unique_id_checksum;
-  BzEntry         *entry               = data->entry;
-  g_autoptr (GError) local_error       = NULL;
-  g_autoptr (BzGuard) slot_guard       = NULL;
-  g_autoptr (BzGuard) other_guard      = NULL;
-  g_autoptr (GMutexLocker) locker      = NULL;
-  guint      slot_queued               = G_MAXUINT;
-  guint      slot_index                = 0;
-  DexFuture *writing_future            = NULL;
-  g_autoptr (LivingEntryData) living   = NULL;
-  g_autoptr (DexPromise) promise       = NULL;
-  g_autoptr (GVariantBuilder) builder  = NULL;
-  g_autoptr (GVariant) variant         = NULL;
-  g_autoptr (GBytes) bytes             = NULL;
-  g_autofree char *main_cache          = NULL;
-  g_autoptr (GFile) parent_file        = NULL;
-  g_autoptr (GFile) save_file          = NULL;
-  g_autoptr (GFileOutputStream) output = NULL;
-  gssize   bytes_written               = 0;
-  gboolean result                      = FALSE;
-  g_autoptr (GError) ret_error         = NULL;
+  OngoingTaskData *task_data              = data->task_data;
+  char            *unique_id_checksum     = data->unique_id_checksum;
+  BzEntry         *entry                  = data->entry;
+  g_autoptr (GError) local_error          = NULL;
+  g_autoptr (BzGuard) slot_guard          = NULL;
+  g_autoptr (BzGuard) other_guard         = NULL;
+  g_autoptr (GMutexLocker) locker         = NULL;
+  guint      slot_queued                  = G_MAXUINT;
+  guint      slot_index                   = 0;
+  DexFuture *writing_future               = NULL;
+  g_autoptr (LivingEntryData) living      = NULL;
+  g_autoptr (DexPromise) promise          = NULL;
+  g_autoptr (GVariantBuilder) builder     = NULL;
+  g_autoptr (GVariant) variant            = NULL;
+  g_autoptr (GBytes) bytes                = NULL;
+  gsize            bytes_size             = 0;
+  gconstpointer    bytes_data             = 0;
+  g_autofree char *main_cache             = NULL;
+  g_autoptr (GFile) parent_file           = NULL;
+  g_autofree char *save_file_path         = NULL;
+  g_autoptr (GFile) save_file             = NULL;
+  gsize            existing_contents_size = 0;
+  g_autofree char *existing_contents      = NULL;
+  g_autoptr (GFileOutputStream) output    = NULL;
+  gssize   bytes_written                  = 0;
+  gboolean result                         = FALSE;
+  g_autoptr (GError) ret_error            = NULL;
 
   if (!BZ_IS_FLATPAK_ENTRY (entry))
     return dex_future_new_reject (
@@ -477,8 +487,9 @@ write_task_fiber (WriteTaskData *data)
   {
     builder = g_variant_builder_new (G_VARIANT_TYPE_VARDICT);
     bz_serializable_serialize (BZ_SERIALIZABLE (entry), builder);
-    variant = g_variant_builder_end (builder);
-    bytes   = g_variant_get_data_as_bytes (variant);
+    variant    = g_variant_builder_end (builder);
+    bytes      = g_variant_get_data_as_bytes (variant);
+    bytes_data = g_bytes_get_data (bytes, &bytes_size);
 
     main_cache  = bz_dup_module_dir ();
     parent_file = g_file_new_for_path (main_cache);
@@ -497,45 +508,55 @@ write_task_fiber (WriteTaskData *data)
             goto done;
           }
       }
-    save_file = g_file_new_build_filename (main_cache, unique_id_checksum, NULL);
+    save_file_path = g_build_filename (main_cache, unique_id_checksum, NULL);
+    save_file      = g_file_new_for_path (save_file_path);
 
-    output = g_file_replace (
-        save_file,
-        NULL,
-        FALSE,
-        G_FILE_CREATE_REPLACE_DESTINATION,
-        NULL,
-        &local_error);
-    if (output == NULL)
+    result = g_file_get_contents (
+        save_file_path, &existing_contents,
+        &existing_contents_size, NULL);
+    /* Only write if the file has definitely changed */
+    if (!result ||
+        existing_contents_size != bytes_size ||
+        memcmp (existing_contents, bytes_data, bytes_size) != 0)
       {
-        ret_error = g_error_new (
-            BZ_ENTRY_CACHE_ERROR,
-            BZ_ENTRY_CACHE_ERROR_CACHE_FAILED,
-            "Failed to open write stream when caching '%s': %s",
-            unique_id_checksum, local_error->message);
-        goto done;
-      }
+        output = g_file_replace (
+            save_file,
+            NULL,
+            FALSE,
+            G_FILE_CREATE_REPLACE_DESTINATION,
+            NULL,
+            &local_error);
+        if (output == NULL)
+          {
+            ret_error = g_error_new (
+                BZ_ENTRY_CACHE_ERROR,
+                BZ_ENTRY_CACHE_ERROR_CACHE_FAILED,
+                "Failed to open write stream when caching '%s': %s",
+                unique_id_checksum, local_error->message);
+            goto done;
+          }
 
-    bytes_written = g_output_stream_write_bytes (G_OUTPUT_STREAM (output), bytes, NULL, &local_error);
-    if (bytes_written < 0)
-      {
-        ret_error = g_error_new (
-            BZ_ENTRY_CACHE_ERROR,
-            BZ_ENTRY_CACHE_ERROR_CACHE_FAILED,
-            "Failed to write data to stream when caching '%s': %s",
-            unique_id_checksum, local_error->message);
-        goto done;
-      }
+        bytes_written = g_output_stream_write_bytes (G_OUTPUT_STREAM (output), bytes, NULL, &local_error);
+        if (bytes_written < 0)
+          {
+            ret_error = g_error_new (
+                BZ_ENTRY_CACHE_ERROR,
+                BZ_ENTRY_CACHE_ERROR_CACHE_FAILED,
+                "Failed to write data to stream when caching '%s': %s",
+                unique_id_checksum, local_error->message);
+            goto done;
+          }
 
-    result = g_output_stream_close (G_OUTPUT_STREAM (output), NULL, &local_error);
-    if (!result)
-      {
-        ret_error = g_error_new (
-            BZ_ENTRY_CACHE_ERROR,
-            BZ_ENTRY_CACHE_ERROR_CACHE_FAILED,
-            "Failed to close stream when caching '%s': %s",
-            unique_id_checksum, local_error->message);
-        goto done;
+        result = g_output_stream_close (G_OUTPUT_STREAM (output), NULL, &local_error);
+        if (!result)
+          {
+            ret_error = g_error_new (
+                BZ_ENTRY_CACHE_ERROR,
+                BZ_ENTRY_CACHE_ERROR_CACHE_FAILED,
+                "Failed to close stream when caching '%s': %s",
+                unique_id_checksum, local_error->message);
+            goto done;
+          }
       }
 
     g_timer_start (living->cached);
@@ -816,13 +837,14 @@ watch_cb (DexFuture       *future,
 static DexFuture *
 watch_work_fiber (OngoingTaskData *task_data)
 {
-  g_autoptr (BzGuard) guard0 = NULL;
-  GHashTableIter iter        = { 0 };
-  g_autoptr (GTimer) timer   = NULL;
-  guint total                = 0;
-  guint active               = 0;
-  guint alive                = 0;
-  guint pruned               = 0;
+  g_autoptr (BzEntryCacheManager) self = NULL;
+  g_autoptr (BzGuard) guard0           = NULL;
+  GHashTableIter iter                  = { 0 };
+  g_autoptr (GTimer) timer             = NULL;
+  guint total                          = 0;
+  guint active                         = 0;
+  guint alive                          = 0;
+  guint pruned                         = 0;
 
   timer = g_timer_new ();
 
@@ -876,7 +898,30 @@ watch_work_fiber (OngoingTaskData *task_data)
            g_timer_elapsed (timer, NULL),
            total, active, alive, pruned, WATCH_CLEANUP_INTERVAL_MSEC);
 
+  bz_weak_get_or_return_reject (self, task_data->self);
+  g_mutex_lock (&self->mutex);
+  self->living_entries = active + alive;
+  g_mutex_unlock (&self->mutex);
+
+  dex_future_disown (dex_scheduler_spawn (
+      dex_scheduler_get_default (),
+      bz_get_dex_stack_size (),
+      (DexFiberFunc) notify_props_fiber,
+      bz_track_weak (self),
+      bz_weak_release));
+
   return dex_timeout_new_msec (WATCH_CLEANUP_INTERVAL_MSEC);
 }
 
+static DexFuture *
+notify_props_fiber (GWeakRef *wr)
+{
+  g_autoptr (BzEntryCacheManager) self = NULL;
+
+  bz_weak_get_or_return_reject (self, wr);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_LIVING_ENTRIES]);
+  return dex_future_new_true ();
+}
+
 /* End of bz-entry-cache-manager.c */
