@@ -43,6 +43,9 @@ bz_entry_group_lock (BzEntryGroup *self);
 GListModel *
 bz_entry_group_get_model (BzEntryGroup *self);
 
+GListModel *
+bz_entry_group_get_installed_versions (BzEntryGroup *self);
+
 const char *
 bz_entry_group_get_id (BzEntryGroup *self);
 
@@ -130,7 +133,8 @@ bz_entry_group_reap_user_data (BzEntryGroup *self);
 void
 bz_entry_group_add (BzEntryGroup *self,
                     BzEntry      *entry,
-                    BzEntry      *runtime);
+                    BzEntry      *runtime,
+                    gboolean      ignore_eol);
 
 void
 bz_entry_group_connect_living (BzEntryGroup *self,
