@@ -73,6 +73,7 @@ typedef struct
 {
   gint     hold;
   gboolean installed;
+  char    *installed_version;
   gboolean searchable;
 
   guint             kinds;
@@ -142,6 +143,7 @@ enum
 
   PROP_HOLDING,
   PROP_INSTALLED,
+  PROP_INSTALLED_VERSION,
   PROP_SEARCHABLE,
   PROP_KINDS,
   PROP_ADDONS,
@@ -297,6 +299,9 @@ bz_entry_get_property (GObject    *object,
     case PROP_INSTALLED:
       g_value_set_boolean (value, priv->installed);
       break;
+    case PROP_INSTALLED_VERSION:
+      g_value_set_string (value, priv->installed_version);
+      break;
     case PROP_SEARCHABLE:
       g_value_set_boolean (value, priv->searchable);
       break;
@@ -486,6 +491,10 @@ bz_entry_set_property (GObject      *object,
     case PROP_INSTALLED:
       priv->installed = g_value_get_boolean (value);
       break;
+    case PROP_INSTALLED_VERSION:
+      g_clear_pointer (&priv->installed_version, g_free);
+      priv->installed_version = g_value_dup_string (value);
+      break;
     case PROP_SEARCHABLE:
       priv->searchable = g_value_get_boolean (value);
       break;
@@ -744,6 +753,12 @@ bz_entry_class_init (BzEntryClass *klass)
           NULL, NULL, FALSE,
           G_PARAM_READWRITE);
 
+  props[PROP_INSTALLED_VERSION] =
+      g_param_spec_string (
+          "installed-version",
+          NULL, NULL, NULL,
+          G_PARAM_READWRITE);
+
   props[PROP_SEARCHABLE] =
       g_param_spec_boolean (
           "searchable",
@@ -1126,6 +1141,8 @@ bz_entry_real_serialize (BzSerializable  *serializable,
   BzEntryPrivate *priv = bz_entry_get_instance_private (self);
 
   g_variant_builder_add (builder, "{sv}", "installed", g_variant_new_boolean (priv->installed));
+  if (priv->installed_version != NULL)
+    g_variant_builder_add (builder, "{sv}", "installed-version", g_variant_new_string (priv->installed_version));
   g_variant_builder_add (builder, "{sv}", "kinds", g_variant_new_uint32 (priv->kinds));
   g_variant_builder_add (builder, "{sv}", "searchable", g_variant_new_boolean (priv->searchable));
   if (priv->addons != NULL)
@@ -1493,6 +1510,8 @@ bz_entry_real_deserialize (BzSerializable *serializable,
 
       if (g_strcmp0 (key, "installed") == 0)
         priv->installed = g_variant_get_boolean (value);
+      else if (g_strcmp0 (key, "installed-version") == 0)
+        priv->installed_version = g_variant_dup_string (value, NULL);
       else if (g_strcmp0 (key, "kinds") == 0)
         priv->kinds = g_variant_get_uint32 (value);
       else if (g_strcmp0 (key, "searchable") == 0)
@@ -1875,6 +1894,31 @@ bz_entry_set_installed (BzEntry *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_INSTALLED]);
 }
 
+const char *
+bz_entry_get_installed_version (BzEntry *self)
+{
+  BzEntryPrivate *priv = NULL;
+
+  g_return_val_if_fail (BZ_IS_ENTRY (self), NULL);
+  priv = bz_entry_get_instance_private (self);
+
+  return priv->installed_version;
+}
+
+void
+bz_entry_set_installed_version (BzEntry    *self,
+                                const char *version)
+{
+  BzEntryPrivate *priv = NULL;
+
+  g_return_if_fail (BZ_IS_ENTRY (self));
+  priv = bz_entry_get_instance_private (self);
+
+  g_clear_pointer (&priv->installed_version, g_free);
+  priv->installed_version = g_strdup (version);
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_INSTALLED_VERSION]);
+}
+
 gboolean
 bz_entry_is_of_kinds (BzEntry *self,
                       guint    kinds)
@@ -2721,7 +2765,6 @@ maybe_save_paintable (BzEntryPrivate  *priv,
 
   if (!BZ_IS_ASYNC_TEXTURE (paintable))
     {
-      g_warning ("Paintable must be of type BzAsyncTexture to be serialized!");
       return FALSE;
     }
 
@@ -2944,6 +2987,7 @@ clear_entry (BzEntry *self)
   g_clear_pointer (&priv->id, g_free);
   g_clear_pointer (&priv->unique_id, g_free);
   g_clear_pointer (&priv->unique_id_checksum, g_free);
+  g_clear_pointer (&priv->installed_version, g_free);
   g_clear_pointer (&priv->title, g_free);
   g_clear_pointer (&priv->eol, g_free);
   g_clear_pointer (&priv->description, g_free);
