@@ -25,14 +25,9 @@
 
 G_BEGIN_DECLS
 
-void
-bz_show_alert_for_widget (GtkWidget  *widget,
-                          const char *title,
-                          const char *text,
-                          gboolean    markup);
-
 void
 bz_show_error_for_widget (GtkWidget  *widget,
+                          const char *title,
                           const char *text);
 
 DexFuture *
