@@ -67,25 +67,10 @@ enum
 
 static guint signals[LAST_SIGNAL];
 
-static gboolean
-test_is_support (BzEntry *entry);
-
-static BzEntry *
-find_entry (BzEntryGroup *group,
-            gboolean (*test) (BzEntry *entry),
-            GtkWidget *window,
-            GError   **error);
-
-static DexFuture *
-install_remove_fiber (BzFavoritesTile *tile);
-
 static void
 install_remove_cb (BzFavoritesTile *self,
                    GtkButton       *button);
 
-static DexFuture *
-support_fiber (BzFavoritesTile *tile);
-
 static void
 support_cb (BzFavoritesTile *self,
             GtkButton       *button);
@@ -185,14 +170,22 @@ get_install_remove_icon (gpointer object,
 }
 
 static gboolean
-switch_bool (gpointer  object,
-             gboolean  condition,
-             gboolean  true_value,
-             gboolean  false_value)
+switch_bool (gpointer object,
+             gboolean condition,
+             gboolean true_value,
+             gboolean false_value)
 {
   return condition ? true_value : false_value;
 }
 
+static gboolean
+logical_and (gpointer object,
+             gboolean value1,
+             gboolean value2)
+{
+  return value1 && value2;
+}
+
 static void
 bz_favorites_tile_class_init (BzFavoritesTileClass *klass)
 {
@@ -239,6 +232,7 @@ bz_favorites_tile_class_init (BzFavoritesTileClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, is_null);
   gtk_widget_class_bind_template_callback (widget_class, is_zero);
   gtk_widget_class_bind_template_callback (widget_class, switch_bool);
+  gtk_widget_class_bind_template_callback (widget_class, logical_and);
   gtk_widget_class_bind_template_callback (widget_class, get_install_remove_tooltip);
   gtk_widget_class_bind_template_callback (widget_class, get_install_remove_icon);
   gtk_widget_class_bind_template_callback (widget_class, install_remove_cb);
@@ -281,144 +275,42 @@ bz_favorites_tile_get_group (BzFavoritesTile *self)
   return self->group;
 }
 
-static gboolean
-test_is_support (BzEntry *entry)
-{
-  return bz_entry_get_donation_url (entry) != NULL;
-}
-
-static BzEntry *
-find_entry (BzEntryGroup *group,
-            gboolean (*test) (BzEntry *entry),
-            GtkWidget *window,
-            GError   **error)
-{
-  g_autoptr (GError) local_error     = NULL;
-  g_autoptr (BzEntry) entry          = NULL;
-  g_autoptr (GListModel) all_entries = NULL;
-
-  entry = bz_entry_group_find_entry (group, test, window, &local_error);
-  if (entry != NULL)
-    return g_steal_pointer (&entry);
-
-  g_clear_error (&local_error);
-
-  all_entries = dex_await_object (
-      bz_entry_group_dup_all_into_store (group),
-      error);
-  if (all_entries == NULL)
-    return NULL;
-
-  if (test == NULL)
-    {
-      if (g_list_model_get_n_items (all_entries) == 0)
-        {
-          g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                       "No entries found in group");
-          return NULL;
-        }
-      return g_list_model_get_item (all_entries, 0);
-    }
-
-  for (guint i = 0; i < g_list_model_get_n_items (all_entries); i++)
-    {
-      g_autoptr (BzEntry) candidate = g_list_model_get_item (all_entries, i);
-
-      if (test (candidate))
-        return g_steal_pointer (&candidate);
-    }
-
-  g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-               "No entry matching criteria found");
-  return NULL;
-}
-
-static DexFuture *
-install_remove_fiber (BzFavoritesTile *tile)
-{
-  g_autoptr (GError) local_error = NULL;
-  g_autoptr (BzEntry) entry      = NULL;
-  BzFavoritesPage *page          = NULL;
-  GtkWidget       *window        = NULL;
-  int removable                  = 0;
-
-  page = BZ_FAVORITES_PAGE (gtk_widget_get_ancestor (GTK_WIDGET (tile), BZ_TYPE_FAVORITES_PAGE));
-  g_assert (page != NULL);
-
-  window = gtk_widget_get_ancestor (GTK_WIDGET (tile), GTK_TYPE_WINDOW);
-  g_assert (window != NULL);
-
-  removable = bz_entry_group_get_removable (tile->group);
-
-  if (removable > 0)
-    entry = bz_entry_group_find_entry (tile->group, NULL, window, &local_error);
-  else
-    entry = find_entry (tile->group, NULL, window, &local_error);
-
-  if (entry == NULL)
-    goto err;
-
-  if (removable > 0)
-    g_signal_emit_by_name (page, "remove", entry);
-  else
-    g_signal_emit_by_name (page, "install", entry);
-
-  return NULL;
-
-err:
-  if (local_error != NULL)
-    bz_show_error_for_widget (window, local_error->message);
-  return NULL;
-}
-
 static void
 install_remove_cb (BzFavoritesTile *self,
                    GtkButton       *button)
 {
-  dex_future_disown (dex_scheduler_spawn (
-      dex_scheduler_get_default (),
-      bz_get_dex_stack_size (),
-      (DexFiberFunc) install_remove_fiber,
-      g_object_ref (self),
-      g_object_unref));
-}
+  BzFavoritesPage *page      = NULL;
+  int              removable = 0;
 
-static DexFuture *
-support_fiber (BzFavoritesTile *tile)
-{
-  g_autoptr (GError) local_error = NULL;
-  g_autoptr (BzEntry) entry      = NULL;
-  GtkWidget  *window             = NULL;
-  const char *url                = NULL;
+  if (self->group == NULL)
+    return;
 
-  window = gtk_widget_get_ancestor (GTK_WIDGET (tile), GTK_TYPE_WINDOW);
-  g_assert (window != NULL);
+  page = BZ_FAVORITES_PAGE (gtk_widget_get_ancestor (GTK_WIDGET (self), BZ_TYPE_FAVORITES_PAGE));
+  if (page == NULL)
+    return;
 
-  entry = find_entry (tile->group, test_is_support, window, &local_error);
-  if (entry == NULL)
-    goto err;
+  removable = bz_entry_group_get_removable (self->group);
 
-  url = bz_entry_get_donation_url (entry);
-  g_app_info_launch_default_for_uri (url, NULL, NULL);
-
-  return NULL;
-
-err:
-  if (local_error != NULL)
-    bz_show_error_for_widget (window, local_error->message);
-  return NULL;
+  if (removable > 0)
+    g_signal_emit_by_name (page, "remove", self->group);
+  else
+    g_signal_emit_by_name (page, "install", self->group);
 }
 
 static void
 support_cb (BzFavoritesTile *self,
             GtkButton       *button)
 {
-  dex_future_disown (dex_scheduler_spawn (
-      dex_scheduler_get_default (),
-      bz_get_dex_stack_size (),
-      (DexFiberFunc) support_fiber,
-      g_object_ref (self),
-      g_object_unref));
+  const char *url = NULL;
+
+  if (self->group == NULL)
+    return;
+
+  url = bz_entry_group_get_donation_url (self->group);
+  if (url == NULL)
+    return;
+
+  g_app_info_launch_default_for_uri (url, NULL, NULL);
 }
 
 static DexFuture *
@@ -465,7 +357,7 @@ unfavorite_fiber (BzFavoritesTile *tile)
       gtk_stack_set_visible_child_name (tile->unfavorite_stack, "button");
       window = gtk_widget_get_ancestor (GTK_WIDGET (tile), GTK_TYPE_WINDOW);
       if (window != NULL)
-        bz_show_error_for_widget (window, local_error->message);
+        bz_show_error_for_widget (window, _("Failed to remove favorite"), local_error->message);
     }
   else
     {
