@@ -27,26 +27,31 @@
 #include <xmlb.h>
 
 #include "bz-app-permissions.h"
+#include "bz-application-map-factory.h"
+#include "bz-application.h"
 #include "bz-appstream-parser.h"
 #include "bz-flatpak-private.h"
 #include "bz-io.h"
+#include "bz-result.h"
 #include "bz-serializable.h"
+#include "bz-state-info.h"
 
 struct _BzFlatpakEntry
 {
   BzEntry parent_instance;
 
-  gboolean user;
-  gboolean is_bundle;
-  gboolean is_installed_ref;
-  char    *flatpak_name;
-  char    *flatpak_id;
-  char    *flatpak_version;
-  char    *application_name;
-  char    *application_runtime;
-  char    *application_command;
-  char    *runtime_name;
-  char    *addon_extension_of_ref;
+  gboolean  user;
+  gboolean  is_bundle;
+  gboolean  is_installed_ref;
+  char     *flatpak_name;
+  char     *flatpak_id;
+  char     *flatpak_version;
+  char     *application_name;
+  char     *application_runtime;
+  char     *application_command;
+  char     *runtime_name;
+  char     *addon_extension_of_ref;
+  BzResult *runtime_result;
 
   FlatpakRef *ref;
 };
@@ -73,6 +78,7 @@ enum
   PROP_APPLICATION_RUNTIME,
   PROP_APPLICATION_COMMAND,
   PROP_RUNTIME_NAME,
+  PROP_RUNTIME_RESULT,
   PROP_ADDON_OF_REF,
 
   LAST_PROP
@@ -109,6 +115,9 @@ bz_flatpak_entry_get_property (GObject    *object,
     case PROP_FLATPAK_ID:
       g_value_set_string (value, self->flatpak_id);
       break;
+    case PROP_FLATPAK_NAME:
+      g_value_set_string (value, self->flatpak_name);
+      break;
     case PROP_FLATPAK_VERSION:
       g_value_set_string (value, self->flatpak_version);
       break;
@@ -127,6 +136,9 @@ bz_flatpak_entry_get_property (GObject    *object,
     case PROP_RUNTIME_NAME:
       g_value_set_string (value, self->runtime_name);
       break;
+    case PROP_RUNTIME_RESULT:
+      g_value_take_object (value, bz_flatpak_entry_dup_runtime_result (self));
+      break;
     case PROP_ADDON_OF_REF:
       g_value_set_string (value, self->addon_extension_of_ref);
       break;
@@ -225,6 +237,13 @@ bz_flatpak_entry_class_init (BzFlatpakEntryClass *klass)
           NULL, NULL, NULL,
           G_PARAM_READABLE);
 
+  props[PROP_RUNTIME_RESULT] =
+      g_param_spec_object (
+          "runtime",
+          NULL, NULL,
+          BZ_TYPE_RESULT,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
   props[PROP_ADDON_OF_REF] =
       g_param_spec_string (
           "addon-extension-of-ref",
@@ -318,6 +337,37 @@ serializable_iface_init (BzSerializableInterface *iface)
   iface->deserialize = bz_flatpak_entry_real_deserialize;
 }
 
+BzResult *
+bz_flatpak_entry_dup_runtime_result (BzFlatpakEntry *self)
+{
+  BzStateInfo             *state             = NULL;
+  BzApplicationMapFactory *factory           = NULL;
+  g_autofree char         *runtime_unique_id = NULL;
+
+  g_return_val_if_fail (BZ_IS_FLATPAK_ENTRY (self), NULL);
+
+  state = bz_state_info_get_default ();
+
+  if (self->runtime_result != NULL)
+    return g_object_ref (self->runtime_result);
+
+  if (self->application_runtime == NULL)
+    return NULL;
+
+  factory = bz_state_info_get_entry_factory (state);
+  if (factory == NULL)
+    return NULL;
+
+  runtime_unique_id = g_strdup_printf ("FLATPAK-SYSTEM::flathub::runtime/%s",
+                                       self->application_runtime);
+
+  self->runtime_result = bz_application_map_factory_convert_one (
+      factory,
+      gtk_string_object_new (runtime_unique_id));
+
+  return self->runtime_result != NULL ? g_object_ref (self->runtime_result) : NULL;
+}
+
 BzFlatpakEntry *
 bz_flatpak_entry_new_for_ref (FlatpakRef    *ref,
                               FlatpakRemote *remote,
@@ -377,6 +427,32 @@ bz_flatpak_entry_new_for_ref (FlatpakRef    *ref,
   }                                         \
   G_STMT_END
 
+  if (!g_log_writer_default_would_drop (G_LOG_LEVEL_DEBUG, G_LOG_DOMAIN))
+    {
+      gsize n_groups        = 0;
+      g_auto (GStrv) groups = NULL;
+
+      g_print ("Debug Key File Data for %s - groups:\n", flatpak_ref_get_name (ref));
+
+      groups = g_key_file_get_groups (key_file, &n_groups);
+      for (gsize i = 0; i < n_groups; i++)
+        {
+          gsize n_keys        = 0;
+          g_auto (GStrv) keys = NULL;
+
+          g_print ("   group %s\n", groups[i]);
+
+          keys = g_key_file_get_keys (key_file, groups[i], &n_keys, NULL);
+          for (gsize j = 0; j < n_keys; j++)
+            {
+              g_autofree char *value = NULL;
+
+              value = g_key_file_get_value (key_file, groups[i], keys[j], NULL);
+              g_print ("     %s=%s\n", keys[j], value);
+            }
+        }
+    }
+
   if (g_key_file_has_group (key_file, "Application"))
     {
       kinds |= BZ_ENTRY_KIND_APPLICATION;
@@ -387,22 +463,18 @@ bz_flatpak_entry_new_for_ref (FlatpakRef    *ref,
         GET_STRING (application_command, "Application", "command");
     }
 
-  if (g_key_file_has_group (key_file, "Runtime"))
-    {
-      if (!g_key_file_has_group (key_file, "Build"))
-        kinds |= BZ_ENTRY_KIND_RUNTIME;
-
-      GET_STRING (runtime_name, "Runtime", "name");
-    }
-
   if (g_key_file_has_group (key_file, "ExtensionOf"))
     {
-      if (!(kinds & BZ_ENTRY_KIND_RUNTIME))
-        kinds |= BZ_ENTRY_KIND_ADDON;
-
+      kinds |= BZ_ENTRY_KIND_ADDON;
       GET_STRING (addon_extension_of_ref, "ExtensionOf", "ref");
     }
 
+  if (g_key_file_has_group (key_file, "Runtime"))
+    {
+      kinds |= BZ_ENTRY_KIND_RUNTIME;
+      GET_STRING (runtime_name, "Runtime", "name");
+    }
+
 #undef GET_STRING
 
   // if (kinds == 0)
@@ -493,32 +565,31 @@ bz_flatpak_entry_new_for_ref (FlatpakRef    *ref,
         }
       else if (FLATPAK_IS_INSTALLED_REF (ref))
         {
-          const char *icon_name = flatpak_ref_get_name (ref);
-          const int   sizes[]   = { 512, 256, 128, 64, 48 };
+          BzStateInfo      *state     = NULL;
+          const char       *icon_name = NULL;
+          GtkIconTheme     *theme     = NULL;
+          GtkIconPaintable *paintable = NULL;
 
-          for (int i = 0; i < G_N_ELEMENTS (sizes); i++)
+          state     = bz_state_info_get_default ();
+          icon_name = flatpak_ref_get_name (ref);
+
+          theme = user
+                      ? bz_state_info_get_user_icon_theme (state)
+                      : bz_state_info_get_system_icon_theme (state);
+
+          if (theme != NULL)
             {
-              g_autofree char *icon_path = NULL;
+              paintable = gtk_icon_theme_lookup_icon (
+                  theme,
+                  icon_name,
+                  NULL,
+                  128,
+                  1,
+                  GTK_TEXT_DIR_NONE,
+                  0);
 
-              if (user)
-                icon_path = g_build_filename (g_get_home_dir (), ".local/share/flatpak/exports/share/icons/hicolor", g_strdup_printf ("%dx%d", sizes[i], sizes[i]), "apps", g_strdup_printf ("%s.png", icon_name), NULL);
-              else
-                icon_path = g_build_filename ("/var/lib/flatpak/exports/share/icons/hicolor", g_strdup_printf ("%dx%d", sizes[i], sizes[i]), "apps", g_strdup_printf ("%s.png", icon_name), NULL);
-
-              if (g_file_test (icon_path, G_FILE_TEST_EXISTS))
-                {
-                  g_autoptr (GFile) icon_file = NULL;
-                  GdkTexture *texture         = NULL;
-
-                  icon_file = g_file_new_for_path (icon_path);
-                  texture   = gdk_texture_new_from_file (icon_file, NULL);
-
-                  if (texture != NULL)
-                    {
-                      icon_paintable = (GdkPaintable *) g_steal_pointer (&texture);
-                      break;
-                    }
-                }
+              if (paintable != NULL)
+                icon_paintable = (GdkPaintable *) g_steal_pointer (&paintable);
             }
         }
     }
@@ -534,6 +605,15 @@ bz_flatpak_entry_new_for_ref (FlatpakRef    *ref,
         title = self->flatpak_id;
     }
 
+  if ((kinds & BZ_ENTRY_KIND_RUNTIME) && !(kinds & BZ_ENTRY_KIND_ADDON) && self->flatpak_version != NULL)
+    {
+      if (!g_str_has_suffix (title, self->flatpak_version)) // GNOME runtimes have the flatpak version at the end whilst others don't.
+        {
+          g_autofree char *original_title = g_strdup (title);
+          title                           = g_strdup_printf ("%s %s", original_title, self->flatpak_version);
+        }
+    }
+
   if (FLATPAK_IS_REMOTE_REF (ref))
     eol = flatpak_remote_ref_get_eol (FLATPAK_REMOTE_REF (ref));
   else if (FLATPAK_IS_INSTALLED_REF (ref))
@@ -744,4 +824,5 @@ clear_entry (BzFlatpakEntry *self)
   g_clear_pointer (&self->application_command, g_free);
   g_clear_pointer (&self->runtime_name, g_free);
   g_clear_pointer (&self->addon_extension_of_ref, g_free);
+  g_clear_object (&self->runtime_result);
 }
