@@ -32,6 +32,7 @@ typedef struct
   /* Template widgets */
   AdwDialog   parent_instance;
   GtkListBox *releases_box;
+  GListModel *installed_versions;
 } BzReleasesDialog;
 
 typedef struct
@@ -48,6 +49,7 @@ struct _BzReleasesList
   AdwBin parent_instance;
 
   GListModel *version_history;
+  GListModel *installed_versions;
 
   /* Template widgets */
   GtkListBox *preview_box;
@@ -60,11 +62,39 @@ enum
 {
   PROP_0,
   PROP_VERSION_HISTORY,
+  PROP_INSTALLED_VERSIONS,
   LAST_PROP
 };
 
 static GParamSpec *props[LAST_PROP] = { 0 };
 
+static gboolean
+is_version_installed (GListModel *installed_versions, const char *version)
+{
+  guint n_items = 0;
+
+  if (!installed_versions || !version)
+    return FALSE;
+
+  n_items = g_list_model_get_n_items (installed_versions);
+  for (guint i = 0; i < n_items; i++)
+    {
+      g_autoptr (GObject) item = NULL;
+
+      item = g_list_model_get_item (installed_versions, i);
+      if (GTK_IS_STRING_OBJECT (item))
+        {
+          const char *installed_version = NULL;
+
+          installed_version = gtk_string_object_get_string (GTK_STRING_OBJECT (item));
+          if (installed_version != NULL && g_strcmp0 (installed_version, version) == 0)
+            return TRUE;
+        }
+    }
+
+  return FALSE;
+}
+
 static char *
 format_timestamp (gpointer object,
                   guint64  value)
@@ -82,21 +112,21 @@ format_timestamp (gpointer object,
     return NULL;
 
   if (g_date_time_get_year (date) == g_date_time_get_year (now))
-    /* Translators: This is a date format for timestamps from previous years. Used in the app releases section.
-     * %B is the full month name, %e is the day, %Y is the year.
-     * Example: "October 1, 2025"
-     * See https://docs.gtk.org/glib/method.DateTime.format.html for format options
-     * Please modify to make it sound natural in your locale.
-     *  */
-    return g_date_time_format (date, "%e %B");
-  else
     /* Translators: This is a date format for timestamps from the current year. Used in the app releases section.
      * %B is the full month name, %e is the day.
      * Example: "October 1"
      * See https://docs.gtk.org/glib/method.DateTime.format.html for format options
      * Please modify to make it sound natural in your locale.
      *  */
-    return g_date_time_format (date, "%e %B %Y");
+    return g_date_time_format (date, N_ ("%e %B"));
+  else
+    /* Translators: This is a date format for timestamps from previous years. Used in the app releases section.
+     * %B is the full month name, %e is the day, %Y is the year.
+     * Example: "October 1, 2025"
+     * See https://docs.gtk.org/glib/method.DateTime.format.html for format options
+     * Please modify to make it sound natural in your locale.
+     *  */
+    return g_date_time_format (date, N_ ("%e %B %Y"));
 }
 
 static GtkWidget *
@@ -104,13 +134,15 @@ create_release_row (const char *version,
                     const char *description,
                     guint64     timestamp,
                     const char *url,
-                    gboolean    use_clamp)
+                    gboolean    use_clamp,
+                    GListModel *installed_versions)
 {
   AdwActionRow                 *row                = NULL;
   GtkBox                       *content_box        = NULL;
   GtkBox                       *header_box         = NULL;
   GtkLabel                     *version_label      = NULL;
   GtkLabel                     *date_label         = NULL;
+  GtkLabel                     *installed_label    = NULL;
   BzAppstreamDescriptionRender *description_widget = NULL;
   BzFadingClamp                *fading_clamp       = NULL;
   GtkBox                       *more_info_box      = NULL;
@@ -131,7 +163,7 @@ create_release_row (const char *version,
   gtk_widget_set_margin_start (GTK_WIDGET (content_box), 15);
   gtk_widget_set_margin_end (GTK_WIDGET (content_box), 15);
 
-  header_box = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0));
+  header_box = GTK_BOX (gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8));
 
   version_text  = g_strdup_printf (_ ("Version %s"), version);
   version_label = GTK_LABEL (gtk_label_new (version_text));
@@ -139,9 +171,22 @@ create_release_row (const char *version,
   gtk_widget_add_css_class (GTK_WIDGET (version_label), "heading");
   gtk_label_set_ellipsize (version_label, PANGO_ELLIPSIZE_END);
   gtk_widget_set_halign (GTK_WIDGET (version_label), GTK_ALIGN_START);
-  gtk_widget_set_hexpand (GTK_WIDGET (version_label), TRUE);
   gtk_box_append (header_box, GTK_WIDGET (version_label));
 
+  if (is_version_installed (installed_versions, version))
+    {
+      installed_label = GTK_LABEL (gtk_label_new (_ ("Installed")));
+      gtk_widget_add_css_class (GTK_WIDGET (installed_label), "lozenge");
+      gtk_widget_add_css_class (GTK_WIDGET (installed_label), "small");
+      gtk_widget_set_halign (GTK_WIDGET (installed_label), GTK_ALIGN_START);
+      gtk_widget_set_hexpand (GTK_WIDGET (installed_label), TRUE);
+      gtk_box_append (header_box, GTK_WIDGET (installed_label));
+    }
+  else
+    {
+      gtk_widget_set_hexpand (GTK_WIDGET (version_label), TRUE);
+    }
+
   date_label = GTK_LABEL (gtk_label_new (date_str ? date_str : ""));
   gtk_widget_add_css_class (GTK_WIDGET (date_label), "dim-label");
   gtk_widget_set_halign (GTK_WIDGET (date_label), GTK_ALIGN_END);
@@ -149,7 +194,7 @@ create_release_row (const char *version,
 
   gtk_box_append (content_box, GTK_WIDGET (header_box));
 
-  if (description && *description)
+  if (description != NULL && *description)
     {
       description_widget = bz_appstream_description_render_new ();
       bz_appstream_description_render_set_appstream_description (description_widget, description);
@@ -170,7 +215,9 @@ create_release_row (const char *version,
     }
   else
     {
-      GtkLabel *fallback_label = GTK_LABEL (gtk_label_new (_ ("No details for this release")));
+      GtkLabel *fallback_label = NULL;
+
+      fallback_label = GTK_LABEL (gtk_label_new (_ ("No details for this release")));
       gtk_widget_set_margin_top (GTK_WIDGET (fallback_label), 5);
       gtk_widget_add_css_class (GTK_WIDGET (fallback_label), "dim-label");
       gtk_label_set_xalign (fallback_label, 0.0);
@@ -200,11 +247,23 @@ create_release_row (const char *version,
   return GTK_WIDGET (row);
 }
 
+static void
+bz_releases_dialog_dispose (GObject *object)
+{
+  BzReleasesDialog *self = (BzReleasesDialog *) object;
+
+  g_clear_object (&self->installed_versions);
+  G_OBJECT_CLASS (bz_releases_dialog_parent_class)->dispose (object);
+}
+
 static void
 bz_releases_dialog_class_init (BzReleasesDialogClass *klass)
 {
+  GObjectClass   *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+  object_class->dispose = bz_releases_dialog_dispose;
+
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/io/github/kolunmi/Bazaar/bz-releases-dialog.ui");
   gtk_widget_class_bind_template_child (widget_class, BzReleasesDialog, releases_box);
@@ -216,15 +275,51 @@ bz_releases_dialog_init (BzReleasesDialog *self)
   gtk_widget_init_template (GTK_WIDGET (self));
 }
 
-static GtkWidget *
-bz_releases_dialog_new (void)
+GtkWidget *
+bz_releases_dialog_new (GListModel *version_history,
+                        GListModel *installed_versions)
 {
-  return g_object_new (bz_releases_dialog_get_type (), NULL);
+  BzReleasesDialog *dialog  = NULL;
+  guint             n_items = 0;
+
+  dialog = g_object_new (bz_releases_dialog_get_type (), NULL);
+
+  if (installed_versions)
+    dialog->installed_versions = g_object_ref (installed_versions);
+
+  if (version_history == NULL)
+    return GTK_WIDGET (dialog);
+
+  n_items = g_list_model_get_n_items (version_history);
+  for (guint i = 0; i < n_items; i++)
+    {
+      g_autoptr (BzRelease) release = NULL;
+      const char *version           = NULL;
+      const char *description       = NULL;
+      const char *url               = NULL;
+      guint64     timestamp         = 0;
+      GtkWidget  *row               = NULL;
+
+      release = g_list_model_get_item (version_history, i);
+      if (release == NULL)
+        continue;
+
+      version     = bz_release_get_version (release);
+      description = bz_release_get_description (release);
+      url         = bz_release_get_url (release);
+      timestamp   = bz_release_get_timestamp (release);
+
+      row = create_release_row (version, description, timestamp, url, FALSE, dialog->installed_versions);
+      gtk_list_box_append (dialog->releases_box, row);
+    }
+
+  return GTK_WIDGET (dialog);
 }
 
-static void
+void
 bz_releases_dialog_set_version_history (BzReleasesDialog *self,
-                                        GListModel       *version_history)
+                                        GListModel       *version_history,
+                                        GListModel       *installed_versions)
 {
   guint      n_items = 0;
   GtkWidget *child   = NULL;
@@ -234,6 +329,10 @@ bz_releases_dialog_set_version_history (BzReleasesDialog *self,
   while ((child = gtk_widget_get_first_child (GTK_WIDGET (self->releases_box))) != NULL)
     gtk_list_box_remove (self->releases_box, child);
 
+  g_clear_object (&self->installed_versions);
+  if (installed_versions)
+    self->installed_versions = g_object_ref (installed_versions);
+
   if (version_history == NULL)
     return;
 
@@ -256,7 +355,7 @@ bz_releases_dialog_set_version_history (BzReleasesDialog *self,
       url         = bz_release_get_url (release);
       timestamp   = bz_release_get_timestamp (release);
 
-      row = create_release_row (version, description, timestamp, url, FALSE);
+      row = create_release_row (version, description, timestamp, url, FALSE, self->installed_versions);
       gtk_list_box_append (self->releases_box, row);
     }
 }
@@ -314,7 +413,7 @@ populate_preview_box (BzReleasesList *self)
           description = bz_release_get_description (release);
           timestamp   = bz_release_get_timestamp (release);
 
-          row = create_release_row (version, description, timestamp, NULL, TRUE);
+          row = create_release_row (version, description, timestamp, NULL, TRUE, self->installed_versions);
           gtk_list_box_insert (self->preview_box, row, 0);
         }
     }
@@ -326,9 +425,8 @@ static void
 show_all_releases_cb (AdwButtonRow   *button,
                       BzReleasesList *self)
 {
-  GtkWidget        *dialog          = NULL;
-  GtkRoot          *root            = NULL;
-  BzReleasesDialog *releases_dialog = NULL;
+  GtkWidget *dialog = NULL;
+  GtkRoot   *root   = NULL;
 
   g_return_if_fail (BZ_IS_RELEASES_LIST (self));
 
@@ -336,9 +434,7 @@ show_all_releases_cb (AdwButtonRow   *button,
   if (root == NULL)
     return;
 
-  dialog          = bz_releases_dialog_new ();
-  releases_dialog = (BzReleasesDialog *) dialog;
-  bz_releases_dialog_set_version_history (releases_dialog, self->version_history);
+  dialog = bz_releases_dialog_new (self->version_history, self->installed_versions);
   adw_dialog_present (ADW_DIALOG (dialog), GTK_WIDGET (root));
 }
 
@@ -348,6 +444,7 @@ bz_releases_list_dispose (GObject *object)
   BzReleasesList *self = BZ_RELEASES_LIST (object);
 
   g_clear_object (&self->version_history);
+  g_clear_object (&self->installed_versions);
 
   G_OBJECT_CLASS (bz_releases_list_parent_class)->dispose (object);
 }
@@ -365,6 +462,9 @@ bz_releases_list_get_property (GObject    *object,
     case PROP_VERSION_HISTORY:
       g_value_set_object (value, self->version_history);
       break;
+    case PROP_INSTALLED_VERSIONS:
+      g_value_set_object (value, self->installed_versions);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -383,6 +483,11 @@ bz_releases_list_set_property (GObject      *object,
     case PROP_VERSION_HISTORY:
       bz_releases_list_set_version_history (self, g_value_get_object (value));
       break;
+    case PROP_INSTALLED_VERSIONS:
+      g_clear_object (&self->installed_versions);
+      self->installed_versions = g_value_dup_object (value);
+      populate_preview_box (self);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -405,6 +510,13 @@ bz_releases_list_class_init (BzReleasesListClass *klass)
                            G_TYPE_LIST_MODEL,
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  props[PROP_INSTALLED_VERSIONS] =
+      g_param_spec_object ("installed-versions",
+                           NULL,
+                           NULL,
+                           G_TYPE_LIST_MODEL,
+                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   g_type_ensure (BZ_TYPE_RELEASE);
