@@ -28,6 +28,7 @@
 #include "bz-rich-app-tile.h"
 #include "bz-screenshot.h"
 #include "bz-search-result.h"
+#include "bz-search-pill-list.h"
 #include "bz-search-widget.h"
 #include "bz-util.h"
 
@@ -241,40 +242,6 @@ no_results_found_subtitle (gpointer    object,
   return g_strdup_printf (_ ("No results found for \"%s\" in Flathub"), search_text);
 }
 
-static void
-action_move (GtkWidget  *widget,
-             const char *action_name,
-             GVariant   *parameter)
-{
-  BzSearchWidget    *self     = BZ_SEARCH_WIDGET (widget);
-  GtkSelectionModel *model    = NULL;
-  guint              selected = 0;
-  guint              n_items  = 0;
-
-  model   = gtk_grid_view_get_model (self->grid_view);
-  n_items = g_list_model_get_n_items (G_LIST_MODEL (model));
-
-  if (n_items == 0)
-    return;
-
-  selected = gtk_single_selection_get_selected (GTK_SINGLE_SELECTION (model));
-
-  if (selected == GTK_INVALID_LIST_POSITION)
-    selected = 0;
-  else
-    {
-      int offset = 0;
-
-      offset = g_variant_get_int32 (parameter);
-      if (offset < 0 && ABS (offset) > selected)
-        selected = n_items + (offset % -(int) n_items);
-      else
-        selected = (selected + offset) % n_items;
-    }
-
-  gtk_widget_activate_action (GTK_WIDGET (self->grid_view), "list.scroll-to-item", "u", selected);
-}
-
 static void
 apps_page_select_cb (BzSearchWidget *self,
                      BzEntryGroup   *group,
@@ -283,6 +250,16 @@ apps_page_select_cb (BzSearchWidget *self,
   g_signal_emit (self, signals[SIGNAL_SELECT], 0, group, FALSE);
 }
 
+static void
+pill_list_cb (BzSearchWidget *self,
+              const char     *label,
+              GtkWidget      *pill_list)
+{
+
+  bz_search_widget_set_text (self, label);
+  update_filter (self);
+}
+
 static void
 category_clicked (BzFlathubCategory *category,
                   GtkButton         *button)
@@ -412,6 +389,7 @@ bz_search_widget_class_init (BzSearchWidgetClass *klass)
   g_type_ensure (BZ_TYPE_RICH_APP_TILE);
   g_type_ensure (BZ_TYPE_SCREENSHOT);
   g_type_ensure (BZ_TYPE_SEARCH_RESULT);
+  g_type_ensure (BZ_TYPE_SEARCH_PILL_LIST);
 
   gtk_widget_class_set_template_from_resource (widget_class, "/io/github/kolunmi/Bazaar/bz-search-widget.ui");
   gtk_widget_class_bind_template_child (widget_class, BzSearchWidget, search_bar);
@@ -430,10 +408,9 @@ bz_search_widget_class_init (BzSearchWidgetClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, idx_to_string);
   gtk_widget_class_bind_template_callback (widget_class, score_to_string);
   gtk_widget_class_bind_template_callback (widget_class, reset_search_cb);
+  gtk_widget_class_bind_template_callback (widget_class, pill_list_cb);
   gtk_widget_class_bind_template_callback (widget_class, no_results_found_subtitle);
   gtk_widget_class_bind_template_callback (widget_class, tile_activated_cb);
-
-  gtk_widget_class_install_action (widget_class, "move", "i", action_move);
 }
 
 static void
@@ -503,6 +480,11 @@ bz_search_widget_set_state (BzSearchWidget *self,
   if (state != NULL)
     {
       self->state = g_object_ref (state);
+      g_signal_connect_swapped (
+          state,
+          "notify::disable-blocklists",
+          G_CALLBACK (invalidating_state_prop_changed),
+          self);
       g_signal_connect_swapped (
           state,
           "notify::hide-eol",
@@ -635,6 +617,9 @@ search_activate (GtkText        *text,
   model   = gtk_grid_view_get_model (self->grid_view);
   n_items = g_list_model_get_n_items (G_LIST_MODEL (model));
 
+  if (gtk_widget_get_visible (GTK_WIDGET (self->search_busy)))
+    return;
+
   if (n_items > 0)
     {
       result = g_list_model_get_item (G_LIST_MODEL (model), 0);
