@@ -139,6 +139,11 @@ bulk_install (BzWindow *self,
 static void
 set_page (BzWindow *self);
 
+static void
+emit_hook_disown (BzWindow     *self,
+                  BzHookSignal  signal,
+                  BzEntryGroup *group);
+
 static void
 bz_window_dispose (GObject *object)
 {
@@ -570,6 +575,7 @@ bz_window_class_init (BzWindowClass *klass)
   gtk_widget_class_install_action (widget_class, "window.open-library", NULL, action_open_library);
 
   gtk_widget_class_add_binding_action (widget_class, GDK_KEY_d, GDK_CONTROL_MASK, "window.open-library", NULL);
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_w, GDK_CONTROL_MASK, "window.close", NULL);
 }
 
 static gboolean
@@ -648,6 +654,7 @@ transact_fiber (TransactData *data)
   g_autoptr (DexFuture) transact_future = NULL;
   g_autofree char *id_dup               = NULL;
   BzMainConfig    *config               = NULL;
+  GListModel      *hooks                = NULL;
   gboolean         delete_user_data     = FALSE;
   GdkDisplay      *display              = NULL;
   GdkSeat         *seat                 = NULL;
@@ -676,46 +683,24 @@ transact_fiber (TransactData *data)
     }
 
   config = bz_state_info_get_main_config (self->state);
-#define RUN_HOOK(_signal)                                              \
-  /* TODO: make reading config less bad */                             \
-  if (config != NULL &&                                                \
-      bz_main_config_get_hooks (config) != NULL)                       \
-    {                                                                  \
-      GListModel *hooks   = NULL;                                      \
-      guint       n_hooks = 0;                                         \
-                                                                       \
-      hooks   = bz_main_config_get_hooks (config);                     \
-      n_hooks = g_list_model_get_n_items (hooks);                      \
-                                                                       \
-      for (guint j = 0; j < n_hooks; j++)                              \
-        {                                                              \
-          g_autoptr (BzHook) hook        = NULL;                       \
-          BzHookSignal       when        = 0;                          \
-          BzHookReturnStatus hook_result = 0;                          \
-                                                                       \
-          hook = g_list_model_get_item (hooks, j);                     \
-          when = bz_hook_get_when (hook);                              \
-                                                                       \
-          if (when == (_signal))                                       \
-            hook_result = dex_await_int (                              \
-                bz_execute_hook (                                      \
-                    hook,                                              \
-                    data->remove                                       \
-                        ? BZ_HOOK_TRANSACTION_TYPE_REMOVAL             \
-                        : BZ_HOOK_TRANSACTION_TYPE_INSTALL,            \
-                    id_dup),                                           \
-                NULL);                                                 \
-                                                                       \
-          if (hook_result == BZ_HOOK_RETURN_STATUS_CONFIRM ||          \
-              hook_result == BZ_HOOK_RETURN_STATUS_STOP)               \
-            break;                                                     \
-          else if (hook_result == BZ_HOOK_RETURN_STATUS_DENY)          \
-            return dex_future_new_reject (                             \
-                BZ_TRANSACTION_MGR_ERROR,                              \
-                BZ_TRANSACTION_MGR_ERROR_CANCELLED_BY_HOOK,            \
-                "The transaction was prevented by a configured hook"); \
-        }                                                              \
-    }
+  if (config != NULL)
+    hooks = bz_main_config_get_hooks (config);
+
+#define RUN_HOOK(_signal)                                               \
+  G_STMT_START                                                          \
+  {                                                                     \
+    if (hooks != NULL &&                                                \
+        !dex_await (                                                    \
+            bz_run_hook_emission (                                      \
+                hooks, (_signal),                                       \
+                data->remove                                            \
+                    ? BZ_HOOK_TRANSACTION_TYPE_REMOVAL                  \
+                    : BZ_HOOK_TRANSACTION_TYPE_INSTALL,                 \
+                id_dup, NULL),                                          \
+            &local_error))                                              \
+      return dex_future_new_for_error (g_steal_pointer (&local_error)); \
+  }                                                                     \
+  G_STMT_END
 
   RUN_HOOK (BZ_HOOK_SIGNAL_BEFORE_TRANSACTION);
 
@@ -877,6 +862,7 @@ bz_window_show_group (BzWindow     *self,
   g_return_if_fail (BZ_IS_ENTRY_GROUP (group));
 
   bz_full_view_set_entry_group (self->full_view, group);
+  emit_hook_disown (self, BZ_HOOK_SIGNAL_VIEW_APP, group);
 
   visible_page = adw_navigation_view_get_visible_page (self->navigation_view);
   if (visible_page != adw_navigation_view_find_page (self->navigation_view, "view"))
@@ -1068,9 +1054,7 @@ search (BzWindow   *self,
         const char *initial)
 {
   if (initial != NULL && *initial != '\0')
-    {
-      bz_search_widget_set_text (self->search_widget, initial);
-    }
+    bz_search_widget_set_text (self->search_widget, initial);
 
   adw_view_stack_set_visible_child_name (self->main_view_stack, "search");
   adw_navigation_view_pop_to_tag (self->navigation_view, "main");
@@ -1091,12 +1075,33 @@ set_page (BzWindow *self)
       adw_navigation_view_pop_to_tag (self->navigation_view, "main");
     }
   else
-    {
-      gtk_stack_set_visible_child_name (self->main_stack, "main");
-    }
+    gtk_stack_set_visible_child_name (self->main_stack, "main");
 
   selected_navigation_page_name = adw_navigation_view_get_visible_page_tag (self->navigation_view);
 
   if (g_strcmp0 (selected_navigation_page_name, "view") != 0)
     bz_full_view_set_entry_group (self->full_view, NULL);
 }
+
+static void
+emit_hook_disown (BzWindow     *self,
+                  BzHookSignal  signal,
+                  BzEntryGroup *group)
+{
+  BzMainConfig *config = NULL;
+  GListModel   *hooks  = NULL;
+
+  if (self->state == NULL)
+    return;
+
+  config = bz_state_info_get_main_config (self->state);
+  if (config == NULL)
+    return;
+
+  hooks = bz_main_config_get_hooks (config);
+  if (hooks == NULL)
+    return;
+
+  dex_future_disown (bz_run_hook_emission (
+      hooks, signal, 0, NULL, group));
+}
